__title__ = 'wotclientdetection'
__version__ = '2024.07.28.1'

from wotclientdetection.manager import LauncherManager
from wotclientdetection.constants import LauncherFlavour, ClientBranch, ClientRealm

__all__ = (
    'LauncherManager',
    'LauncherFlavour',
    'ClientBranch',
    'ClientBranch',
    'ClientRealm',
)
